{
    "$graph": [
        {
            "class": "Workflow",
            "label": "GATK Base Recalibration on Bam",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement"
                },
                {
                    "class": "StepInputExpressionRequirement"
                },
                {
                    "class": "MultipleInputFeatureRequirement"
                }
            ],
            "inputs": [
                {
                    "id": "#main/bam",
                    "type": "File",
                    "secondaryFiles": [
                        ".bai"
                    ]
                },
                {
                    "id": "#main/intervals",
                    "doc": "This optional interval supports processing by regions. If this input resolves to null, then GATK will process the whole genome per each tool's spec",
                    "type": [
                        "File",
                        "null"
                    ]
                },
                {
                    "id": "#main/reference",
                    "type": "File",
                    "secondaryFiles": [
                        ".fai",
                        ".amb",
                        ".ann",
                        ".bwt",
                        ".pac",
                        ".sa",
                        "^.dict"
                    ]
                },
                {
                    "id": "#main/snps_dbsnp",
                    "type": "File",
                    "secondaryFiles": [
                        ".tbi"
                    ]
                },
                {
                    "id": "#main/snps_1000gp",
                    "type": "File",
                    "secondaryFiles": [
                        ".tbi"
                    ]
                },
                {
                    "id": "#main/known_indels",
                    "type": "File",
                    "secondaryFiles": [
                        ".tbi"
                    ]
                },
                {
                    "id": "#main/mills_indels",
                    "type": "File",
                    "secondaryFiles": [
                        ".tbi"
                    ]
                }
            ],
            "steps": [
                {
                    "id": "#main/base_recalibrator",
                    "label": "GATK4: Base Recalibrator",
                    "in": [
                        {
                            "id": "#main/base_recalibrator/bam",
                            "source": "#main/bam"
                        },
                        {
                            "id": "#main/base_recalibrator/knownSites",
                            "source": [
                                "#main/snps_dbsnp",
                                "#main/snps_1000gp",
                                "#main/known_indels",
                                "#main/mills_indels"
                            ]
                        },
                        {
                            "id": "#main/base_recalibrator/reference",
                            "source": "#main/reference"
                        },
                        {
                            "id": "#main/base_recalibrator/intervals",
                            "source": "#main/intervals"
                        }
                    ],
                    "run": "#Gatk4BaseRecalibrator_4_1_3_0.cwl",
                    "out": [
                        {
                            "id": "#main/base_recalibrator/out"
                        }
                    ]
                },
                {
                    "id": "#main/apply_bqsr",
                    "label": "GATK4: Apply base quality score recalibration",
                    "in": [
                        {
                            "id": "#main/apply_bqsr/bam",
                            "source": "#main/bam"
                        },
                        {
                            "id": "#main/apply_bqsr/reference",
                            "source": "#main/reference"
                        },
                        {
                            "id": "#main/apply_bqsr/recalFile",
                            "source": "#main/base_recalibrator/out"
                        },
                        {
                            "id": "#main/apply_bqsr/intervals",
                            "source": "#main/intervals"
                        }
                    ],
                    "run": "#Gatk4ApplyBQSR_4_1_3_0.cwl",
                    "out": [
                        {
                            "id": "#main/apply_bqsr/out"
                        }
                    ]
                }
            ],
            "id": "#main",
            "outputs": [
                {
                    "id": "#main/out",
                    "type": "File",
                    "secondaryFiles": [
                        ".bai"
                    ],
                    "outputSource": "#main/apply_bqsr/out"
                }
            ]
        },
        {
            "class": "CommandLineTool",
            "label": "GATK4: Apply base quality score recalibration",
            "doc": "Apply base quality score recalibration: This tool performs the second pass in a two-stage \nprocess called Base Quality Score Recalibration (BQSR). Specifically, it recalibrates the \nbase qualities of the input reads based on the recalibration table produced by the \nBaseRecalibrator tool, and outputs a recalibrated BAM or CRAM file.\n\nSummary of the BQSR procedure: The goal of this procedure is to correct for systematic bias \nthat affect the assignment of base quality scores by the sequencer. The first pass consists \nof calculating error empirically and finding patterns in how error varies with basecall \nfeatures over all bases. The relevant observations are written to a recalibration table. \nThe second pass consists of applying numerical corrections to each individual basecall \nbased on the patterns identified in the first step (recorded in the recalibration table) \nand write out the recalibrated data to a new BAM or CRAM file.\n\n- This tool replaces the use of PrintReads for the application of base quality score \n    recalibration as practiced in earlier versions of GATK (2.x and 3.x).\n- You should only run ApplyBQSR with the covariates table created from the input BAM or CRAM file(s).\n- Original qualities can be retained in the output file under the \"OQ\" tag if desired. \n    See the `--emit-original-quals` argument for details.",
            "requirements": [
                {
                    "class": "ShellCommandRequirement"
                },
                {
                    "class": "InlineJavascriptRequirement"
                },
                {
                    "class": "DockerRequirement",
                    "dockerPull": "broadinstitute/gatk:4.1.3.0"
                }
            ],
            "inputs": [
                {
                    "id": "#Gatk4ApplyBQSR_4_1_3_0.cwl/Gatk4ApplyBQSR/javaOptions",
                    "label": "javaOptions",
                    "type": [
                        {
                            "type": "array",
                            "items": "string"
                        },
                        "null"
                    ]
                },
                {
                    "id": "#Gatk4ApplyBQSR_4_1_3_0.cwl/Gatk4ApplyBQSR/compression_level",
                    "label": "compression_level",
                    "doc": "Compression level for all compressed files created (e.g. BAM and VCF). Default value: 2.",
                    "type": [
                        "int",
                        "null"
                    ]
                },
                {
                    "id": "#Gatk4ApplyBQSR_4_1_3_0.cwl/Gatk4ApplyBQSR/bam",
                    "label": "bam",
                    "doc": "The SAM/BAM/CRAM file containing reads.",
                    "type": "File",
                    "secondaryFiles": [
                        "${\n\n        function resolveSecondary(base, secPattern) {\n          if (secPattern[0] == \"^\") {\n            var spl = base.split(\".\");\n            var endIndex = spl.length > 1 ? spl.length - 1 : 1;\n            return resolveSecondary(spl.slice(undefined, endIndex).join(\".\"), secPattern.slice(1));\n          }\n          return base + secPattern\n        }\n\n        return [\n                {\n                    location: resolveSecondary(self.location, \"^.bai\"),\n                    basename: resolveSecondary(self.basename, \".bai\"),\n                    class: \"File\",\n                }\n        ];\n\n}"
                    ],
                    "inputBinding": {
                        "prefix": "-I",
                        "position": 10
                    }
                },
                {
                    "id": "#Gatk4ApplyBQSR_4_1_3_0.cwl/Gatk4ApplyBQSR/reference",
                    "label": "reference",
                    "doc": "Reference sequence",
                    "type": "File",
                    "secondaryFiles": [
                        ".fai",
                        ".amb",
                        ".ann",
                        ".bwt",
                        ".pac",
                        ".sa",
                        "^.dict"
                    ],
                    "inputBinding": {
                        "prefix": "-R"
                    }
                },
                {
                    "id": "#Gatk4ApplyBQSR_4_1_3_0.cwl/Gatk4ApplyBQSR/outputFilename",
                    "label": "outputFilename",
                    "doc": "Write output to this file",
                    "type": [
                        "string",
                        "null"
                    ],
                    "default": "generated.recalibrated.bam",
                    "inputBinding": {
                        "prefix": "-O",
                        "valueFrom": "$(inputs.bam.basename.replace(/.bam$/, \"\")).recalibrated.bam"
                    }
                },
                {
                    "id": "#Gatk4ApplyBQSR_4_1_3_0.cwl/Gatk4ApplyBQSR/recalFile",
                    "label": "recalFile",
                    "doc": "Input recalibration table for BQSR",
                    "type": [
                        "File",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--bqsr-recal-file"
                    }
                },
                {
                    "id": "#Gatk4ApplyBQSR_4_1_3_0.cwl/Gatk4ApplyBQSR/intervals",
                    "label": "intervals",
                    "doc": "-L (BASE) One or more genomic intervals over which to operate",
                    "type": [
                        "File",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--intervals"
                    }
                },
                {
                    "id": "#Gatk4ApplyBQSR_4_1_3_0.cwl/Gatk4ApplyBQSR/intervalStrings",
                    "label": "intervalStrings",
                    "doc": "-L (BASE) One or more genomic intervals over which to operate",
                    "type": [
                        {
                            "type": "array",
                            "inputBinding": {
                                "prefix": "--intervals"
                            },
                            "items": "string"
                        },
                        "null"
                    ],
                    "inputBinding": {}
                },
                {
                    "id": "#Gatk4ApplyBQSR_4_1_3_0.cwl/Gatk4ApplyBQSR/tmpDir",
                    "label": "tmpDir",
                    "doc": "Temp directory to use.",
                    "type": "string",
                    "default": "/tmp/",
                    "inputBinding": {
                        "prefix": "--tmp-dir",
                        "position": 11
                    }
                }
            ],
            "outputs": [
                {
                    "id": "#Gatk4ApplyBQSR_4_1_3_0.cwl/Gatk4ApplyBQSR/out",
                    "label": "out",
                    "type": "File",
                    "secondaryFiles": [
                        "${\n\n        function resolveSecondary(base, secPattern) {\n          if (secPattern[0] == \"^\") {\n            var spl = base.split(\".\");\n            var endIndex = spl.length > 1 ? spl.length - 1 : 1;\n            return resolveSecondary(spl.slice(undefined, endIndex).join(\".\"), secPattern.slice(1));\n          }\n          return base + secPattern\n        }\n        return [\n                {\n                    path: resolveSecondary(self.path, \"^.bai\"),\n                    basename: resolveSecondary(self.basename, \".bai\"),\n                    class: \"File\",\n                }\n        ];\n\n}"
                    ],
                    "outputBinding": {
                        "glob": "$(inputs.bam.basename.replace(/.bam$/, \"\")).recalibrated.bam",
                        "loadContents": false
                    }
                }
            ],
            "stdout": "_stdout",
            "stderr": "_stderr",
            "baseCommand": [
                "gatk",
                "ApplyBQSR"
            ],
            "arguments": [
                {
                    "prefix": "--java-options",
                    "position": -1,
                    "valueFrom": "$(\"-Xmx{memory}G {compression} {otherargs}\".replace(/\\{memory\\}/g, (([inputs.runtime_memory, 8, 4].filter(function (inner) { return inner != null })[0] * 3) / 4)).replace(/\\{compression\\}/g, (inputs.compression_level != null) ? (\"-Dsamjdk.compress_level=\" + inputs.compression_level) : \"\").replace(/\\{otherargs\\}/g, [inputs.javaOptions, []].filter(function (inner) { return inner != null })[0].join(\" \")))"
                }
            ],
            "id": "#Gatk4ApplyBQSR_4_1_3_0.cwl"
        },
        {
            "class": "CommandLineTool",
            "label": "GATK4: Base Recalibrator",
            "doc": "First pass of the base quality score recalibration. Generates a recalibration table based on various covariates. \nThe default covariates are read group, reported quality score, machine cycle, and nucleotide context.\n\nThis walker generates tables based on specified covariates. It does a by-locus traversal operating only at sites \nthat are in the known sites VCF. ExAc, gnomAD, or dbSNP resources can be used as known sites of variation. \nWe assume that all reference mismatches we see are therefore errors and indicative of poor base quality. \nSince there is a large amount of data one can then calculate an empirical probability of error given the \nparticular covariates seen at this site, where p(error) = num mismatches / num observations. The output file is a \ntable (of the several covariate values, num observations, num mismatches, empirical quality score).",
            "requirements": [
                {
                    "class": "ShellCommandRequirement"
                },
                {
                    "class": "InlineJavascriptRequirement"
                },
                {
                    "class": "DockerRequirement",
                    "dockerPull": "broadinstitute/gatk:4.1.3.0"
                }
            ],
            "inputs": [
                {
                    "id": "#Gatk4BaseRecalibrator_4_1_3_0.cwl/Gatk4BaseRecalibrator/javaOptions",
                    "label": "javaOptions",
                    "type": [
                        {
                            "type": "array",
                            "items": "string"
                        },
                        "null"
                    ]
                },
                {
                    "id": "#Gatk4BaseRecalibrator_4_1_3_0.cwl/Gatk4BaseRecalibrator/compression_level",
                    "label": "compression_level",
                    "doc": "Compression level for all compressed files created (e.g. BAM and VCF). Default value: 2.",
                    "type": [
                        "int",
                        "null"
                    ]
                },
                {
                    "id": "#Gatk4BaseRecalibrator_4_1_3_0.cwl/Gatk4BaseRecalibrator/tmpDir",
                    "label": "tmpDir",
                    "doc": "Temp directory to use.",
                    "type": "string",
                    "default": "/tmp/",
                    "inputBinding": {
                        "prefix": "--tmp-dir"
                    }
                },
                {
                    "id": "#Gatk4BaseRecalibrator_4_1_3_0.cwl/Gatk4BaseRecalibrator/bam",
                    "label": "bam",
                    "doc": "BAM/SAM/CRAM file containing reads",
                    "type": "File",
                    "secondaryFiles": [
                        "${\n\n        function resolveSecondary(base, secPattern) {\n          if (secPattern[0] == \"^\") {\n            var spl = base.split(\".\");\n            var endIndex = spl.length > 1 ? spl.length - 1 : 1;\n            return resolveSecondary(spl.slice(undefined, endIndex).join(\".\"), secPattern.slice(1));\n          }\n          return base + secPattern\n        }\n\n        return [\n                {\n                    location: resolveSecondary(self.location, \"^.bai\"),\n                    basename: resolveSecondary(self.basename, \".bai\"),\n                    class: \"File\",\n                }\n        ];\n\n}"
                    ],
                    "inputBinding": {
                        "prefix": "-I",
                        "position": 6
                    }
                },
                {
                    "id": "#Gatk4BaseRecalibrator_4_1_3_0.cwl/Gatk4BaseRecalibrator/knownSites",
                    "label": "knownSites",
                    "doc": "**One or more databases of known polymorphic sites used to exclude regions around known polymorphisms from analysis.** This algorithm treats every reference mismatch as an indication of error. However, real genetic variation is expected to mismatch the reference, so it is critical that a database of known polymorphic sites is given to the tool in order to skip over those sites. This tool accepts any number of Feature-containing files (VCF, BCF, BED, etc.) for use as this database. For users wishing to exclude an interval list of known variation simply use -XL my.interval.list to skip over processing those sites. Please note however that the statistics reported by the tool will not accurately reflected those sites skipped by the -XL argument.",
                    "type": {
                        "type": "array",
                        "inputBinding": {
                            "prefix": "--known-sites"
                        },
                        "items": "File"
                    },
                    "inputBinding": {
                        "position": 28
                    }
                },
                {
                    "id": "#Gatk4BaseRecalibrator_4_1_3_0.cwl/Gatk4BaseRecalibrator/reference",
                    "label": "reference",
                    "doc": "Reference sequence file",
                    "type": "File",
                    "secondaryFiles": [
                        ".fai",
                        ".amb",
                        ".ann",
                        ".bwt",
                        ".pac",
                        ".sa",
                        "^.dict"
                    ],
                    "inputBinding": {
                        "prefix": "-R",
                        "position": 5
                    }
                },
                {
                    "id": "#Gatk4BaseRecalibrator_4_1_3_0.cwl/Gatk4BaseRecalibrator/outputFilename",
                    "label": "outputFilename",
                    "doc": "**The output recalibration table filename to create.** After the header, data records occur one per line until the end of the file. The first several items on a line are the values of the individual covariates and will change depending on which covariates were specified at runtime. The last three items are the data- that is, number of observations for this combination of covariates, number of reference mismatches, and the raw empirical quality score calculated by phred-scaling the mismatch rate. Use '/dev/stdout' to print to standard out.",
                    "type": [
                        "string",
                        "null"
                    ],
                    "default": "generated.table",
                    "inputBinding": {
                        "prefix": "-O",
                        "position": 8,
                        "valueFrom": "$(inputs.bam.basename.replace(/.bam$/, \"\")).table"
                    }
                },
                {
                    "id": "#Gatk4BaseRecalibrator_4_1_3_0.cwl/Gatk4BaseRecalibrator/intervals",
                    "label": "intervals",
                    "doc": "-L (BASE) One or more genomic intervals over which to operate",
                    "type": [
                        "File",
                        "null"
                    ],
                    "inputBinding": {
                        "prefix": "--intervals"
                    }
                },
                {
                    "id": "#Gatk4BaseRecalibrator_4_1_3_0.cwl/Gatk4BaseRecalibrator/intervalStrings",
                    "label": "intervalStrings",
                    "doc": "-L (BASE) One or more genomic intervals over which to operate",
                    "type": [
                        {
                            "type": "array",
                            "inputBinding": {
                                "prefix": "--intervals"
                            },
                            "items": "string"
                        },
                        "null"
                    ],
                    "inputBinding": {}
                }
            ],
            "outputs": [
                {
                    "id": "#Gatk4BaseRecalibrator_4_1_3_0.cwl/Gatk4BaseRecalibrator/out",
                    "label": "out",
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(inputs.bam.basename.replace(/.bam$/, \"\")).table",
                        "loadContents": false
                    }
                }
            ],
            "stdout": "_stdout",
            "stderr": "_stderr",
            "baseCommand": [
                "gatk",
                "BaseRecalibrator"
            ],
            "arguments": [
                {
                    "prefix": "--java-options",
                    "position": -1,
                    "valueFrom": "$(\"-Xmx{memory}G {compression} {otherargs}\".replace(/\\{memory\\}/g, (([inputs.runtime_memory, 16, 4].filter(function (inner) { return inner != null })[0] * 3) / 4)).replace(/\\{compression\\}/g, (inputs.compression_level != null) ? (\"-Dsamjdk.compress_level=\" + inputs.compression_level) : \"\").replace(/\\{otherargs\\}/g, [inputs.javaOptions, []].filter(function (inner) { return inner != null })[0].join(\" \")))"
                }
            ],
            "id": "#Gatk4BaseRecalibrator_4_1_3_0.cwl"
        }
    ],
    "cwlVersion": "v1.0"
}
